#include "preferences.h"
#include "tooldialog.h"

#include <QFileDialog>
#include <QPushButton>
#include <QStandardItemModel>


Preferences::Preferences(QWidget *parent)
   :QDialog(parent)
{
    setupUi(this);

    mToolModel = new QStandardItemModel(this);

    toolView->setModel(mToolModel);

    buttonBox->button(QDialogButtonBox::Ok)->setText(tr("&OK"));
    buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("&Cancel"));

    connect(toolView->selectionModel(), SIGNAL(currentChanged(QModelIndex, QModelIndex)),
            this, SLOT(currentToolChanged(QModelIndex, QModelIndex)));
}

void Preferences::setTools(const QList<Tool> &tools)
{
    foreach (const Tool& t, tools)
    {
        QList<QStandardItem*> item;

        item.append(new QStandardItem(t.Name));
        item.append(new QStandardItem(t.Command));
        item.append(new QStandardItem(t.Filter));

        mToolModel->appendRow(item);
    }
}

const QList<Tool> Preferences::getTools() const
{
    QList<Tool> list;

    for (int i=0, c=mToolModel->rowCount(); i<c; i++)
    {
        list.append(Tool(
                        mToolModel->index(i, 0).data(Qt::DisplayRole).toString(),
                        mToolModel->index(i, 1).data(Qt::DisplayRole).toString(),
                        mToolModel->index(i, 2).data(Qt::DisplayRole).toString()));
    }

    return list;
}

/* --------------------------------------------------------------------- */

void Preferences::on_browseCmd_clicked()
{
    const QString file = QFileDialog::getOpenFileName(this, tr("Find GVIM executable"), vimCommand->text());

    if (!file.isEmpty())
    {
        vimCommand->setText(file);
    }
}

void Preferences::on_addTool_clicked()
{
    ToolDialog dlg(this);

    if (dlg.exec() == QDialog::Accepted)
    {
        QList<QStandardItem*> item;

        item.append(new QStandardItem(dlg.name->text()));
        item.append(new QStandardItem(dlg.command->text()));
        item.append(new QStandardItem(dlg.filter->text()));

        mToolModel->appendRow(item);
    }
}

void Preferences::on_removeTool_clicked()
{
    if (toolView->currentIndex().isValid())
    {
        mToolModel->removeRow(toolView->currentIndex().row());

        currentToolChanged(toolView->currentIndex());
    }
}

void Preferences::on_moveToolUp_clicked()
{
    if (toolView->currentIndex().isValid())
    {
        const int row = toolView->currentIndex().row();

        if (row > 0)
        {
            mToolModel->insertRow(row - 1, mToolModel->takeRow(row));

            toolView->selectionModel()->setCurrentIndex(mToolModel->index(row - 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void Preferences::on_moveToolDown_clicked()
{
    if (toolView->currentIndex().isValid())
    {
        const int row = toolView->currentIndex().row();

        if (row < mToolModel->rowCount() - 1)
        {
            mToolModel->insertRow(row + 1, mToolModel->takeRow(row));

            toolView->selectionModel()->setCurrentIndex(mToolModel->index(row + 1, 0),
                QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current | QItemSelectionModel::Rows);
        }
    }
}

void Preferences::currentToolChanged(const QModelIndex &index, const QModelIndex &previous)
{
    Q_UNUSED(previous);

    if (index.isValid())
    {
        removeTool->setEnabled(true);
        moveToolUp->setEnabled(index.row() > 0);
        moveToolDown->setEnabled(index.row() < mToolModel->rowCount() - 1);
    }
    else
    {
        removeTool->setEnabled(false);
        moveToolUp->setEnabled(false);
        moveToolDown->setEnabled(false);
    }
}

void Preferences::on_toolView_activated(const QModelIndex &index)
{
    ToolDialog dlg(this);

    QModelIndex name = mToolModel->index(index.row(), 0),
                cmd = mToolModel->index(index.row(), 1),
                filt = mToolModel->index(index.row(), 2);

    dlg.name->setText(name.data().toString());
    dlg.command->setText(cmd.data().toString());
    dlg.filter->setText(filt.data().toString());

    if (dlg.exec() == QDialog::Accepted)
    {
        mToolModel->setData(name, dlg.name->text(), Qt::DisplayRole);
        mToolModel->setData(cmd, dlg.command->text(), Qt::DisplayRole);
        mToolModel->setData(filt, dlg.filter->text(), Qt::DisplayRole);
    }
}
